@@ -3,7 +3,7 @@
 //  SLiM
 //
 //  Created by Ben Haller on 3/27/2020.
-//  Copyright (c) 2020-2024 Philipp Messer.  All rights reserved.
+//  Copyright (c) 2020-2025 Philipp Messer.  All rights reserved.
 //	A product of the Messer Lab, http://messerlab.org/slim/
 //
 
@@ -19,10 +19,10 @@
 
 #include "QtSLiMGraphView_1DPopulationSFS.h"
 
-#include <string>
-
 #include "QtSLiMWindow.h"
 
+#include <string>
+
 
 QtSLiMGraphView_1DPopulationSFS::QtSLiMGraphView_1DPopulationSFS(QWidget *p_parent, QtSLiMWindow *controller) : QtSLiMGraphView(p_parent, controller)
 {
@@ -84,37 +84,25 @@ double *QtSLiMGraphView_1DPopulationSFS::populationSFS(int mutationTypeCount)
 	
 	// get the selected chromosome range
     Species *graphSpecies = focalDisplaySpecies();
-	bool hasSelection;
-	slim_position_t selectionFirstBase;
-	slim_position_t selectionLastBase;
-    controller_->chromosomeSelection(graphSpecies, &hasSelection, &selectionFirstBase, &selectionLastBase);
 	
 	// tally into our bins
 	Population &pop = graphSpecies->population_;
 	
-	pop.TallyMutationReferencesAcrossPopulation(false);	// update tallies; usually this will just use the cache set up by Population::MaintainRegistry()
+	pop.TallyMutationReferencesAcrossPopulation(/* p_clock_for_mutrun_experiments */ false);	// update tallies; usually this will just use the cache set up by Population::MaintainRegistry()
 	
 	Mutation *mut_block_ptr = gSLiM_Mutation_Block;
 	slim_refcount_t *refcount_block_ptr = gSLiM_Mutation_Refcounts;
-	double totalGenomeCount = ((pop.total_genome_count_ == 0) ? 1 : pop.total_genome_count_);   // prevent a zero count from producing NAN frequencies below
     int registry_size;
     const MutationIndex *registry = pop.MutationRegistry(&registry_size);
 	
 	for (int registry_index = 0; registry_index < registry_size; ++registry_index)
 	{
 		const Mutation *mutation = mut_block_ptr + registry[registry_index];
-		
-		// if the user has selected a subrange of the chromosome, we will work from that
-		if (hasSelection)
-		{
-			slim_position_t mutationPosition = mutation->position_;
-			
-			if ((mutationPosition < selectionFirstBase) || (mutationPosition > selectionLastBase))
-				continue;
-		}
+        Chromosome *mut_chromosome = graphSpecies->Chromosomes()[mutation->chromosome_index_];
+        double totalHaplosomeCount = ((mut_chromosome->total_haplosome_count_ == 0) ? 1 : mut_chromosome->total_haplosome_count_);   // prevent a zero count from producing NAN frequencies below
 		
 		slim_refcount_t mutationRefCount = *(refcount_block_ptr + mutation->BlockIndex());
-		double mutationFrequency = mutationRefCount / totalGenomeCount;
+		double mutationFrequency = mutationRefCount / totalHaplosomeCount;
 		int mutationBin = static_cast<int>(floor(mutationFrequency * binCount));
 		int mutationTypeIndex = mutation->mutation_type_ptr_->mutation_type_index_;
 		
@@ -157,29 +145,6 @@ void QtSLiMGraphView_1DPopulationSFS::drawGraph(QPainter &painter, QRect interio
 	
 	// plot our histogram bars
 	drawGroupedBarplot(painter, interiorRect, spectrum, mutationTypeCount, binCount, 0.0, (1.0 / binCount));
-	
-	// if we have a limited selection range, overdraw a note about that
-    bool hasSelection;
-	slim_position_t selectionFirstBase;
-	slim_position_t selectionLastBase;
-    controller_->chromosomeSelection(graphSpecies, &hasSelection, &selectionFirstBase, &selectionLastBase);
-	
-	if (hasSelection)
-	{
-        painter.setFont(QtSLiMGraphView::fontForTickLabels());
-        painter.setBrush(Qt::darkGray);
-		
-        QString labelText = QString("%1 – %2").arg(selectionFirstBase).arg(selectionLastBase);
-        QRect labelBoundingRect = painter.boundingRect(QRect(), Qt::TextDontClip | Qt::TextSingleLine, labelText);
-		double labelX = interiorRect.x() + (interiorRect.width() - labelBoundingRect.width()) / 2.0;
-		double labelY = interiorRect.y() + interiorRect.height() - (labelBoundingRect.height() + 4);
-		
-        labelY = painter.transform().map(QPointF(labelX, labelY)).y();
-        
-        painter.setWorldMatrixEnabled(false);
-        painter.drawText(QPointF(labelX, labelY), labelText);
-        painter.setWorldMatrixEnabled(true);
-	}
 }
 
 QtSLiMLegendSpec QtSLiMGraphView_1DPopulationSFS::legendKey(void)
@@ -187,12 +152,6 @@ QtSLiMLegendSpec QtSLiMGraphView_1DPopulationSFS::legendKey(void)
 	return mutationTypeLegendKey();     // we use the prefab mutation type legend
 }
 
-void QtSLiMGraphView_1DPopulationSFS::controllerChromosomeSelectionChanged(void)
-{
-    invalidateCachedData();
-    update();
-}
-
 bool QtSLiMGraphView_1DPopulationSFS::providesStringForData(void)
 {
     return true;
@@ -202,14 +161,6 @@ void QtSLiMGraphView_1DPopulationSFS::appendStringForData(QString &string)
 {
     // get the selected chromosome range
     Species *graphSpecies = focalDisplaySpecies();
-	bool hasSelection;
-	slim_position_t selectionFirstBase;
-	slim_position_t selectionLastBase;
-    controller_->chromosomeSelection(graphSpecies, &hasSelection, &selectionFirstBase, &selectionLastBase);
-
-	if (hasSelection)
-        string.append(QString("# Selected chromosome range: %1 – %2\n").arg(selectionFirstBase).arg(selectionLastBase));
-	
 	int binCount = histogramBinCount_;
 	int mutationTypeCount = static_cast<int>(graphSpecies->mutation_types_.size());
 	double *plotData = populationSFS(mutationTypeCount);
