@@ -3,7 +3,7 @@
 //  SLiM
 //
 //  Created by Ben Haller on 8/20/2020.
-//  Copyright (c) 2020-2024 Philipp Messer.  All rights reserved.
+//  Copyright (c) 2020-2025 Philipp Messer.  All rights reserved.
 //	A product of the Messer Lab, http://messerlab.org/slim/
 //
 
@@ -37,7 +37,7 @@
 
 QtSLiMGraphView_1DSampleSFS::QtSLiMGraphView_1DSampleSFS(QWidget *p_parent, QtSLiMWindow *controller) : QtSLiMGraphView(p_parent, controller)
 {
-    histogramBinCount_ = 20;        // this is also the genome sample size
+    histogramBinCount_ = 20;        // this is also the haplosome sample size
     allowBinCountRescale_ = false;
     
     x0_ = 0;
@@ -143,7 +143,7 @@ QString QtSLiMGraphView_1DSampleSFS::graphTitle(void)
 
 QString QtSLiMGraphView_1DSampleSFS::aboutString(void)
 {
-    return "The 1D Sample SFS graph shows a Site Frequency Spectrum (SFS) for a sample of genomes taken "
+    return "The 1D Sample SFS graph shows a Site Frequency Spectrum (SFS) for a sample of haplosomes taken "
            "(with replacement) from a given subpopulation, for mutations of a given mutation type.  The x axis "
            "here is the occurrence count of a given mutation within the sample, from 1 to the sample size.  The "
            "y axis is the number of mutations in the sample with that specific occurrence count, on a log "
@@ -274,15 +274,21 @@ uint64_t *QtSLiMGraphView_1DSampleSFS::mutation1DSFS(void)
         
         // Get frequencies for a sample taken (with replacement) from subpop1
         {
-            std::vector<Genome *> sampleGenomes;
-            std::vector<Genome *> &subpopGenomes = subpop1->CurrentGenomes();
-            size_t subpopGenomeCount = subpopGenomes.size();
+            std::vector<Haplosome *> sampleHaplosomes;
+            std::vector<Individual *> &subpopIndividuals = subpop1->parent_individuals_;
+            size_t subpopHaplosomeCount = subpopIndividuals.size() * 2;
             
-            if (subpopGenomeCount)
+            if (subpopHaplosomeCount)
                 for (int i = 0; i < histogramBinCount_ - 1; ++i)
-                    sampleGenomes.emplace_back(subpopGenomes[random() % subpopGenomeCount]);
+                {
+                    slim_popsize_t haplosome_index = random() % subpopHaplosomeCount;
+                    slim_popsize_t individual_index = haplosome_index >> 1;
+                    Haplosome *haplosome = subpopIndividuals[individual_index]->haplosomes_[haplosome_index & 0x01];
+                    
+                    sampleHaplosomes.emplace_back(haplosome);
+                }
             
-            tallyGUIMutationReferences(sampleGenomes, selectedMutationTypeIndex_);
+            tallyGUIMutationReferences(sampleHaplosomes, selectedMutationTypeIndex_);
         }
         
         // Tally into our bins
