@@ -3,7 +3,7 @@
 //  SLiM
 //
 //  Created by Ben Haller on 1/19/2024.
-//  Copyright (c) 2024 Philipp Messer.  All rights reserved.
+//  Copyright (c) 2024-2025 Philipp Messer.  All rights reserved.
 //	A product of the Messer Lab, http://messerlab.org/slim/
 //
 
@@ -27,6 +27,10 @@
 
 #include "QtSLiM_Plot.h"
 
+#include <limits>
+#include <algorithm>
+#include <vector>
+
 
 QtSLiMGraphView_CustomPlot::QtSLiMGraphView_CustomPlot(QWidget *p_parent, QtSLiMWindow *controller) : QtSLiMGraphView(p_parent, controller)
 {
