@@ -3,7 +3,7 @@
 //  SLiM
 //
 //  Created by Ben Haller on 4/4/2020.
-//  Copyright (c) 2020-2024 Philipp Messer.  All rights reserved.
+//  Copyright (c) 2020-2025 Philipp Messer.  All rights reserved.
 //	A product of the Messer Lab, http://messerlab.org/slim/
 //
 
@@ -35,7 +35,7 @@ public:
     explicit QtSLiMHaplotypeProgress(QWidget *p_parent = nullptr);
     virtual ~QtSLiMHaplotypeProgress() override;
     
-    void runProgressWithGenomeCount(size_t genome_count, int stepCount);
+    void runProgressWithHaplosomeCount(size_t haplosome_count, int stepCount, int progressChromIndex, int progressChromTotal);
     bool haplotypeProgressIsCancelled(void);
     
     void setHaplotypeProgress(size_t progress, int stage);
