@@ -3,7 +3,7 @@
 //  SLiM
 //
 //  Created by Ben Haller on 11/19/2019.
-//  Copyright (c) 2019-2024 Philipp Messer.  All rights reserved.
+//  Copyright (c) 2019-2025 Philipp Messer.  All rights reserved.
 //	A product of the Messer Lab, http://messerlab.org/slim/
 //
 
@@ -149,6 +149,7 @@ void QtSLiMHelpOutlineDelegate::paint(QPainter *painter, const QStyleOptionViewI
                               "– addCloned()",
                               "– addCrossed()",
                               "– addEmpty()",
+                              "– addMultiRecombinant()",
                               "– addRecombinant()",
                               "– addSelfed()",
                               "– removeSubpopulation()",
@@ -297,7 +298,7 @@ QtSLiMHelpWindow::QtSLiMHelpWindow(QWidget *p_parent) : QWidget(p_parent, Qt::Wi
     {
         const std::string &element_type = class_object->ClassName();
         
-        if (!Eidos_string_hasPrefix(element_type, "_") && (element_type != "DictionaryBase"))		// internal classes are undocumented
+        if (!Eidos_string_hasPrefix(element_type, "_") && (element_type != "DictionaryRetained"))		// internal classes are undocumented
         {
             checkDocumentationOfClass(class_object);
             addSuperclassItemForClass(class_object);
@@ -509,9 +510,7 @@ void QtSLiMHelpWindow::searchScopeToggled(void)
 void QtSLiMHelpWindow::enterSearchForString(QString searchString, bool titlesOnly)
 {
 	// Show our window and bring it front
-    show();
-    raise();
-    activateWindow();
+    QtSLiMMakeWindowVisibleAndExposed(this);
 	
 	// Set the search string per the request
     ui->searchField->setText(searchString);
@@ -919,9 +918,9 @@ void QtSLiMHelpWindow::checkDocumentationOfClass(EidosClass *classObject)
 {
     const EidosClass *superclassObject = classObject->Superclass();
 	
-	// We're hiding DictionaryBase, where the Dictionary stuff is actually defined, so we have Dictionary pretend that its superclass is Object, so the Dictionary stuff gets checked
-	if (classObject == gEidosDictionaryRetained_Class)
-		superclassObject = gEidosObject_Class;
+	// We're hiding DictionaryRetained, so DictionaryRetained subclasses pretend their superclass is Dictionary
+	if (superclassObject == gEidosDictionaryRetained_Class)
+		superclassObject = gEidosDictionaryUnretained_Class;
 	
     const QString className = QString::fromStdString(classObject->ClassName());
 	const QString classKey = "Class " + className;
@@ -1028,9 +1027,9 @@ void QtSLiMHelpWindow::addSuperclassItemForClass(EidosClass *classObject)
 {
     const EidosClass *superclassObject = classObject->Superclass();
 	
-	// We're hiding DictionaryBase, where the Dictionary stuff is actually defined, so we have Dictionary pretend that its superclass is Object
-	if (classObject == gEidosDictionaryRetained_Class)
-		superclassObject = gEidosObject_Class;
+	// We're hiding DictionaryRetained, so DictionaryRetained subclasses pretend their superclass is Dictionary
+	if (superclassObject == gEidosDictionaryRetained_Class)
+		superclassObject = gEidosDictionaryUnretained_Class;
 	
     const QString className = QString::fromStdString(classObject->ClassName());
 	const QString classKey = "Class " + className;
@@ -1041,10 +1040,6 @@ void QtSLiMHelpWindow::addSuperclassItemForClass(EidosClass *classObject)
 	{
         QString superclassName = superclassObject ? QString::fromStdString(superclassObject->ClassName()) : QString("none");
         
-        // Hide DictionaryBase by pretending our superclass is Dictionary
-        if (superclassName == "DictionaryBase")
-            superclassName = "Dictionary";
-        
         bool inDarkMode = QtSLiMInDarkMode();
         QtSLiMHelpItem *superclassItem = new QtSLiMHelpItem((QTreeWidgetItem *)nullptr);
         
