@@ -3,7 +3,7 @@
 //  SLiM
 //
 //  Created by Ben Haller on 8/3/2019.
-//  Copyright (c) 2019-2024 Philipp Messer.  All rights reserved.
+//  Copyright (c) 2019-2025 Philipp Messer.  All rights reserved.
 //	A product of the Messer Lab, http://messerlab.org/slim/
 //
 
@@ -41,6 +41,8 @@ static const char *QtSLiMDisplayFontSize = "QtSLiMDisplayFontSize";
 static const char *QtSLiMSyntaxHighlightScript = "QtSLiMSyntaxHighlightScript";
 static const char *QtSLiMSyntaxHighlightOutput = "QtSLiMSyntaxHighlightOutput";
 static const char *QtSLiMShowLineNumbers = "QtSLiMShowLineNumbers";
+static const char *QtSLiMShowPageGuide = "QtSLiMShowPageGuide";
+static const char *QtSLiMPageGuideColumn = "QtSLiMPageGuideColumn";
 static const char *QtSLiMHighlightCurrentLine = "QtSLiMHighlightCurrentLine";
 static const char *QtSLiMAutosaveOnRecycle = "QtSLiMAutosaveOnRecycle";
 static const char *QtSLiMShowSaveInUntitled = "QtSLiMShowSaveInUntitled";
@@ -132,8 +134,15 @@ bool QtSLiMPreferencesNotifier::useOpenGLPref(void)
 {
 #ifndef SLIM_NO_OPENGL
     QSettings settings;
-    
+
+#ifdef _WIN32
+    // BCH 3/23/2025: Too many people are getting bitten by OpenGL not working properly on
+    // Windows, for reasons that have not been diagnosed.  Turning this off on Windows.
+    // It runs a little slower, but there will be a lot less confusion.
+    return settings.value(QtSLiMUseOpenGL, QVariant(false)).toBool();
+#else
     return settings.value(QtSLiMUseOpenGL, QVariant(true)).toBool();
+#endif
 #else
     return false;
 #endif
@@ -194,6 +203,20 @@ bool QtSLiMPreferencesNotifier::highlightCurrentLinePref(void) const
     return settings.value(QtSLiMHighlightCurrentLine, QVariant(true)).toBool();
 }
 
+bool QtSLiMPreferencesNotifier::showPageGuidePref(void) const
+{
+    QSettings settings;
+    
+    return settings.value(QtSLiMShowPageGuide, QVariant(false)).toBool();
+}
+
+int QtSLiMPreferencesNotifier::pageGuideColumnPref(void) const
+{
+    QSettings settings;
+    
+    return settings.value(QtSLiMPageGuideColumn, QVariant(80)).toInt();
+}
+
 bool QtSLiMPreferencesNotifier::autosaveOnRecyclePref(void) const
 {
     QSettings settings;
@@ -368,6 +391,25 @@ void QtSLiMPreferencesNotifier::highlightCurrentLineToggled()
     emit highlightCurrentLinePrefChanged();
 }
 
+void QtSLiMPreferencesNotifier::showPageGuideToggled()
+{
+    QtSLiMPreferences &prefsUI = QtSLiMPreferences::instance();
+    QSettings settings;
+    
+    settings.setValue(QtSLiMShowPageGuide, QVariant(prefsUI.ui->showPageGuide->isChecked()));
+    
+    emit pageGuidePrefsChanged();
+}
+
+void QtSLiMPreferencesNotifier::pageGuideColumnChanged(int newColumn)
+{
+    QSettings settings;
+    
+    settings.setValue(QtSLiMPageGuideColumn, QVariant(newColumn));
+    
+    emit pageGuidePrefsChanged();
+}
+
 void QtSLiMPreferencesNotifier::autosaveOnRecycleToggled()
 {
     QtSLiMPreferences &prefsUI = QtSLiMPreferences::instance();
@@ -454,6 +496,11 @@ QtSLiMPreferences::QtSLiMPreferences(QWidget *p_parent) : QDialog(p_parent), ui(
     ui->showLineNumbers->setChecked(notifier->showLineNumbersPref());
     ui->highlightCurrentLine->setChecked(notifier->highlightCurrentLinePref());
     
+    // the presence of this hidden widget fixes a padding bug; see https://forum.qt.io/topic/10757/unwanted-padding-around-qhboxlayout
+    ui->pageGuideNoPadWidget->hide();
+    ui->showPageGuide->setChecked(notifier->showPageGuidePref());
+    ui->pageGuideSpinBox->setValue(notifier->pageGuideColumnPref());
+    
     ui->autosaveOnRecycle->setChecked(notifier->autosaveOnRecyclePref());
     ui->showSaveIfUntitled->setChecked(notifier->showSaveIfUntitledPref());
     ui->showSaveIfUntitled->setEnabled(notifier->autosaveOnRecyclePref());
@@ -472,6 +519,8 @@ QtSLiMPreferences::QtSLiMPreferences(QWidget *p_parent) : QDialog(p_parent), ui(
     
     connect(ui->showLineNumbers, &QCheckBox::toggled, notifier, &QtSLiMPreferencesNotifier::showLineNumbersToggled);
     connect(ui->highlightCurrentLine, &QCheckBox::toggled, notifier, &QtSLiMPreferencesNotifier::highlightCurrentLineToggled);
+    connect(ui->showPageGuide, &QCheckBox::toggled, notifier, &QtSLiMPreferencesNotifier::showPageGuideToggled);
+    connect(ui->pageGuideSpinBox, QOverload<int>::of(&QSpinBox::valueChanged), notifier, &QtSLiMPreferencesNotifier::pageGuideColumnChanged);
     
     connect(ui->autosaveOnRecycle, &QCheckBox::toggled, notifier, &QtSLiMPreferencesNotifier::autosaveOnRecycleToggled);
     connect(ui->showSaveIfUntitled, &QCheckBox::toggled, notifier, &QtSLiMPreferencesNotifier::showSaveIfUntitledToggled);
