@@ -3,7 +3,7 @@
 //  SLiM
 //
 //  Created by Ben Haller on 11/2/20.
-//  Copyright (c) 2020-2024 Philipp Messer.  All rights reserved.
+//  Copyright (c) 2020-2025 Philipp Messer.  All rights reserved.
 //	A product of the Messer Lab, http://messerlab.org/slim/
 //
 
@@ -118,8 +118,8 @@ EidosValue_SP LogFile::_GeneratedValue_PopulationSexRatio(const LogFileGenerator
 		for (auto &subpop_iter : species->population_.subpops_)
 		{
 			Subpopulation *subpop = subpop_iter.second;
-			slim_popsize_t subpop_size = subpop->CurrentSubpopSize();
-			slim_popsize_t first_male_index = subpop->CurrentFirstMaleIndex();
+			slim_popsize_t subpop_size = subpop->parent_subpop_size_;
+			slim_popsize_t first_male_index = subpop->parent_first_male_index_;
 			
 			total_individuals += subpop_size;
 			total_males += (subpop_size - first_male_index);
@@ -143,7 +143,7 @@ EidosValue_SP LogFile::_GeneratedValue_PopulationSize(const LogFileGeneratorInfo
 	slim_popsize_t total_individuals = 0;
 	
 	for (auto &subpop_iter : species->population_.subpops_)
-		total_individuals += (subpop_iter.second)->CurrentSubpopSize();
+		total_individuals += (subpop_iter.second)->parent_subpop_size_;
 	
 	return EidosValue_SP(new (gEidosValuePool->AllocateChunk()) EidosValue_Int(total_individuals));
 }
@@ -154,8 +154,8 @@ EidosValue_SP LogFile::_GeneratedValue_SubpopulationSexRatio(const LogFileGenera
 	
 	if (subpop && subpop->species_.SexEnabled())
 	{
-		slim_popsize_t subpop_size = subpop->CurrentSubpopSize();
-		slim_popsize_t first_male_index = subpop->CurrentFirstMaleIndex();
+		slim_popsize_t subpop_size = subpop->parent_subpop_size_;
+		slim_popsize_t first_male_index = subpop->parent_first_male_index_;
 		double sex_ratio = (subpop_size == 0) ? 0.0 : ((subpop_size - first_male_index) / (double)subpop_size);
 		return EidosValue_SP(new (gEidosValuePool->AllocateChunk()) EidosValue_Float(sex_ratio));
 	}
@@ -172,7 +172,7 @@ EidosValue_SP LogFile::_GeneratedValue_SubpopulationSize(const LogFileGeneratorI
 	
 	if (subpop)
 	{
-		slim_popsize_t subpop_size = subpop->CurrentSubpopSize();
+		slim_popsize_t subpop_size = subpop->parent_subpop_size_;
 		return EidosValue_SP(new (gEidosValuePool->AllocateChunk()) EidosValue_Int(subpop_size));
 	}
 	else
@@ -197,7 +197,7 @@ EidosValue_SP LogFile::_GeneratedValue_CustomScript(const LogFileGeneratorInfo &
 	
 	EidosScript *generator_script = p_generator_info.script_;
 	EidosErrorContext error_context_save = gEidosErrorContext;
-	gEidosErrorContext = EidosErrorContext{{-1, -1, -1, -1}, generator_script, true};
+	gEidosErrorContext = EidosErrorContext{{-1, -1, -1, -1}, generator_script};
 	
 	EidosValue_SP result_SP;
 	
@@ -220,7 +220,16 @@ EidosValue_SP LogFile::_GeneratedValue_CustomScript(const LogFileGeneratorInfo &
 	catch (...)
 	{
 		if (gEidosTerminateThrows)
-			gEidosErrorContext = error_context_save;
+		{
+			// In some cases, such as if the error occurred in a derived user-defined function, we can
+			// actually get a user script error context at this point, and don't need to intervene.
+			if (!gEidosErrorContext.currentScript || (gEidosErrorContext.currentScript->UserScriptUTF16Offset() == -1))
+			{
+				gEidosErrorContext = error_context_save;
+				TranslateErrorContextToUserScript("_GeneratedValue_CustomScript()");
+			}
+		}
+		
 		throw;
 	}
 	
@@ -236,7 +245,7 @@ void LogFile::_GeneratedValues_CustomMeanAndSD(const LogFileGeneratorInfo &p_gen
 	
 	EidosScript *generator_script = p_generator_info.script_;
 	EidosErrorContext error_context_save = gEidosErrorContext;
-	gEidosErrorContext = EidosErrorContext{{-1, -1, -1, -1}, generator_script, true};
+	gEidosErrorContext = EidosErrorContext{{-1, -1, -1, -1}, generator_script};
 	
 	EidosValue_SP result_SP;
 	
@@ -283,7 +292,16 @@ void LogFile::_GeneratedValues_CustomMeanAndSD(const LogFileGeneratorInfo &p_gen
 	catch (...)
 	{
 		if (gEidosTerminateThrows)
-			gEidosErrorContext = error_context_save;
+		{
+			// In some cases, such as if the error occurred in a derived user-defined function, we can
+			// actually get a user script error context at this point, and don't need to intervene.
+			if (!gEidosErrorContext.currentScript || (gEidosErrorContext.currentScript->UserScriptUTF16Offset() == -1))
+			{
+				gEidosErrorContext = error_context_save;
+				TranslateErrorContextToUserScript("_GeneratedValues_CustomMeanAndSD()");
+			}
+		}
+		
 		throw;
 	}
 	
@@ -316,6 +334,13 @@ void LogFile::AppendNewRow(void)
 {
 	THREAD_SAFETY_IN_ACTIVE_PARALLEL("LogFile::AppendNewRow(): filesystem write");
 	
+	// Guarantee that we are in the parent generation for all generators, so they don't need to worry
+	const std::vector<Species *> &all_species = community_.AllSpecies();
+	
+	for (Species *species : all_species)
+		if (species->population_.child_generation_valid_)
+			EIDOS_TERMINATION << "ERROR (LogFile::AppendNewRow): (internal error) generating logfile entry with child generation active!" << EidosTerminate();
+	
 	std::vector<const std::string *> line_vec;
 	std::string header_line;
 	std::string row_line;
@@ -515,7 +540,7 @@ const EidosClass *LogFile::Class(void) const
 
 void LogFile::Print(std::ostream &p_ostream) const
 {
-	p_ostream << Class()->ClassName() << "<" << user_file_path_ << ">";
+	p_ostream << Class()->ClassNameForDisplay() << "<" << user_file_path_ << ">";
 }
 
 EidosValue_SP LogFile::GetProperty(EidosGlobalStringID p_property_id)
@@ -632,9 +657,9 @@ EidosValue_SP LogFile::ExecuteMethod_addCustomColumn(EidosGlobalStringID p_metho
 	
 	// See, e.g., Subpopulation::ApplyFitnessEffectCallbacks() for comments on parsing/running script blocks
 	EidosErrorContext error_context_save = gEidosErrorContext;
-	EidosScript *source_script = new EidosScript(source, -1);
+	EidosScript *source_script = new EidosScript(source);
 	
-	gEidosErrorContext = EidosErrorContext{{-1, -1, -1, -1}, source_script, true};
+	gEidosErrorContext = EidosErrorContext{{-1, -1, -1, -1}, source_script};
 	
 	try {
 		source_script->Tokenize();
@@ -643,7 +668,10 @@ EidosValue_SP LogFile::ExecuteMethod_addCustomColumn(EidosGlobalStringID p_metho
 	catch (...)
 	{
 		if (gEidosTerminateThrows)
+		{
 			gEidosErrorContext = error_context_save;
+			TranslateErrorContextToUserScript("ExecuteMethod_addCustomColumn()");
+		}
 		
 		delete source_script;
 		source_script = nullptr;
@@ -653,6 +681,19 @@ EidosValue_SP LogFile::ExecuteMethod_addCustomColumn(EidosGlobalStringID p_metho
 	
 	gEidosErrorContext = error_context_save;
 	
+	// Check contextValue for validity and make a copy of it.  Copying is needed to
+	// ensure that the value is not changed underneath us externally, for example
+	// by a for loop; see https://github.com/MesserLab/SLiM/issues/496.
+	if (context_value->Type() == EidosValueType::kValueObject)
+	{
+		EidosValue_Object *context_object = (EidosValue_Object *)context_value.get();
+		
+		if (!context_object->Class()->UsesRetainRelease())
+			EIDOS_TERMINATION << "ERROR (LogFile::ExecuteMethod_addCustomColumn): the context parameter to addCustomColumn() cannot be an object of a class that is not under retain-release, since the lifetime of such objects cannot be guaranteed.  See the documentation for addCustomColumn() for discussion of this limitation." << EidosTerminate();
+	}
+	
+	context_value = context_value->CopyValues();
+	
 	generator_info_.emplace_back(LogFileGeneratorType::kGenerator_CustomScript, source_script, -1, std::move(context_value));
 	column_names_.emplace_back(column_name);
 	
@@ -714,9 +755,9 @@ EidosValue_SP LogFile::ExecuteMethod_addMeanSDColumns(EidosGlobalStringID p_meth
 	const std::string &source = source_value->StringRefAtIndex_NOCAST(0, nullptr);
 	
 	EidosErrorContext error_context_save = gEidosErrorContext;
-	EidosScript *source_script = new EidosScript(source, -1);
+	EidosScript *source_script = new EidosScript(source);
 	
-	gEidosErrorContext = EidosErrorContext{{-1, -1, -1, -1}, source_script, true};
+	gEidosErrorContext = EidosErrorContext{{-1, -1, -1, -1}, source_script};
 	
 	try {
 		source_script->Tokenize();
@@ -725,7 +766,10 @@ EidosValue_SP LogFile::ExecuteMethod_addMeanSDColumns(EidosGlobalStringID p_meth
 	catch (...)
 	{
 		if (gEidosTerminateThrows)
+		{
 			gEidosErrorContext = error_context_save;
+			TranslateErrorContextToUserScript("ExecuteMethod_addMeanSDColumns()");
+		}
 		
 		delete source_script;
 		source_script = nullptr;
@@ -735,6 +779,19 @@ EidosValue_SP LogFile::ExecuteMethod_addMeanSDColumns(EidosGlobalStringID p_meth
 	
 	gEidosErrorContext = error_context_save;
 	
+	// Check contextValue for validity and make a copy of it.  Copying is needed to
+	// ensure that the value is not changed underneath us externally, for example
+	// by a for loop; see https://github.com/MesserLab/SLiM/issues/496.
+	if (context_value->Type() == EidosValueType::kValueObject)
+	{
+		EidosValue_Object *context_object = (EidosValue_Object *)context_value.get();
+		
+		if (!context_object->Class()->UsesRetainRelease())
+			EIDOS_TERMINATION << "ERROR (LogFile::ExecuteMethod_addMeanSDColumns): the context parameter to addMeanSDColumns() cannot be an object of a class that is not under retain-release, since the lifetime of such objects cannot be guaranteed.  See the documentation for addCustomColumn() for discussion of this limitation." << EidosTerminate();
+	}
+	
+	context_value = context_value->CopyValues();
+	
 	generator_info_.emplace_back(LogFileGeneratorType::kGenerator_CustomMeanAndSD, source_script, -1, std::move(context_value));
 	column_names_.emplace_back(column_name + "_mean");
 	column_names_.emplace_back(column_name + "_sd");
@@ -813,13 +870,7 @@ EidosValue_SP LogFile::ExecuteMethod_addSubpopulationSexRatio(EidosGlobalStringI
 	}
 	else
 	{
-#if DEBUG
-		// Use dynamic_cast<> only in DEBUG since it is hella slow
-		// the class of the object here should be guaranteed by the caller anyway
-		Subpopulation *subpop = dynamic_cast<Subpopulation *>(subpop_value->ObjectElementAtIndex_NOCAST(0, nullptr));
-#else
 		Subpopulation *subpop = (Subpopulation *)(subpop_value->ObjectElementAtIndex_NOCAST(0, nullptr));
-#endif
 		
 		subpop_id = subpop->subpopulation_id_;
 	}
@@ -847,13 +898,7 @@ EidosValue_SP LogFile::ExecuteMethod_addSubpopulationSize(EidosGlobalStringID p_
 	}
 	else
 	{
-#if DEBUG
-		// Use dynamic_cast<> only in DEBUG since it is hella slow
-		// the class of the object here should be guaranteed by the caller anyway
-		Subpopulation *subpop = dynamic_cast<Subpopulation *>(subpop_value->ObjectElementAtIndex_NOCAST(0, nullptr));
-#else
 		Subpopulation *subpop = (Subpopulation *)(subpop_value->ObjectElementAtIndex_NOCAST(0, nullptr));
-#endif
 		
 		subpop_id = subpop->subpopulation_id_;
 	}
@@ -1106,17 +1151,36 @@ const std::vector<EidosMethodSignature_CSP> *LogFile_Class::Methods(void) const
 		methods->emplace_back((EidosInstanceMethodSignature *)(new EidosInstanceMethodSignature(gStr_willAutolog, kEidosValueMaskLogical | kEidosValueMaskSingleton)));
 		
 		// overrides of Dictionary methods; these should not be declared again, to avoid a duplicate in the methods table
-		//methods->emplace_back((EidosInstanceMethodSignature *)(new EidosInstanceMethodSignature(gEidosStr_addKeysAndValuesFrom, kEidosValueMaskVOID))->AddObject_S(gEidosStr_source, nullptr));
-		//methods->emplace_back((EidosInstanceMethodSignature *)(new EidosInstanceMethodSignature(gEidosStr_appendKeysAndValuesFrom, kEidosValueMaskVOID))->AddObject(gEidosStr_source, nullptr));
+		//methods->emplace_back((EidosInstanceMethodSignature *)(new EidosInstanceMethodSignature(gEidosStr_addKeysAndValuesFrom, kEidosValueMaskVOID))->AddObject_S(gEidosStr_source, gEidosDictionaryUnretained_Class));
+		//methods->emplace_back((EidosInstanceMethodSignature *)(new EidosInstanceMethodSignature(gEidosStr_appendKeysAndValuesFrom, kEidosValueMaskVOID))->AddObject(gEidosStr_source, gEidosDictionaryUnretained_Class));
 		//methods->emplace_back((EidosInstanceMethodSignature *)(new EidosInstanceMethodSignature(gEidosStr_clearKeysAndValues, kEidosValueMaskVOID)));
 		//methods->emplace_back(((EidosInstanceMethodSignature *)(new EidosInstanceMethodSignature(gEidosStr_setValue, kEidosValueMaskVOID))->AddArg(kEidosValueMaskInt | kEidosValueMaskString | kEidosValueMaskSingleton, "key", nullptr)->AddAny("value")));
 		
+		//methods->emplace_back(((EidosInstanceMethodSignature *)(new EidosClassMethodSignature(gEidosStr_setValuesVectorized, kEidosValueMaskVOID))->AddArg(kEidosValueMaskInt | kEidosValueMaskString | kEidosValueMaskSingleton, "key", nullptr)->AddAny("value")));
+		
 		std::sort(methods->begin(), methods->end(), CompareEidosCallSignatures);
 	}
 	
 	return methods;
 }
 
+EidosValue_SP LogFile_Class::ExecuteClassMethod(EidosGlobalStringID p_method_id, EidosValue_Object *p_target, const std::vector<EidosValue_SP> &p_arguments, EidosInterpreter &p_interpreter) const
+{
+	switch (p_method_id)
+	{
+		case gEidosID_setValuesVectorized:	return ExecuteMethod_setValuesVectorized(p_method_id, p_target, p_arguments, p_interpreter);
+		default:							return super::ExecuteClassMethod(p_method_id, p_target, p_arguments, p_interpreter);
+	}
+}
+
+//	*********************	+ (void)setValuesVectorized(is$ key, * values)
+//
+EidosValue_SP LogFile_Class::ExecuteMethod_setValuesVectorized(EidosGlobalStringID p_method_id, EidosValue_Object *p_target, const std::vector<EidosValue_SP> &p_arguments, EidosInterpreter &p_interpreter) const
+{
+#pragma unused (p_method_id, p_target, p_arguments, p_interpreter)
+	EIDOS_TERMINATION << "ERROR (LogFile::ExecuteMethod_setValuesVectorized): LogFile manages its dictionary entries; they cannot be modified by the user." << EidosTerminate(nullptr);
+}
+
 
 
 
@@ -1148,6 +1212,5 @@ const std::vector<EidosMethodSignature_CSP> *LogFile_Class::Methods(void) const
 
 
 
-
 
 
