@@ -3,7 +3,7 @@
 //  SLiM
 //
 //  Created by Ben Haller on 12/13/14.
-//  Copyright (c) 2014-2024 Philipp Messer.  All rights reserved.
+//  Copyright (c) 2014-2025 Philipp Messer.  All rights reserved.
 //	A product of the Messer Lab, http://messerlab.org/slim/
 //
 
@@ -86,7 +86,7 @@ public:
 	EidosValue_SP cached_value_muttype_id_;		// a cached value for mutation_type_id_; reset() if that changes
 	
 	slim_selcoeff_t dominance_coeff_;			// dominance coefficient (h)
-	slim_selcoeff_t haploid_dominance_coeff_;	// dominance coefficient (h) used when one genome is null
+	slim_selcoeff_t hemizygous_dominance_coeff_;	// dominance coefficient (h) used when one haplosome is null
 	
 	DFEType dfe_type_;							// distribution of fitness effects (DFE) type (f: fixed, g: gamma, e: exponential, n: normal, w: Weibull)
 	std::vector<double> dfe_parameters_;		// DFE parameters, of type double (originally float or integer type)
