@@ -3,7 +3,7 @@
 //  SLiM
 //
 //  Created by Ben Haller on 12/13/14.
-//  Copyright (c) 2014-2024 Philipp Messer.  All rights reserved.
+//  Copyright (c) 2014-2025 Philipp Messer.  All rights reserved.
 //	A product of the Messer Lab, http://messerlab.org/slim/
 //
 
@@ -55,7 +55,9 @@ public:
 	Polymorphism(void) = delete;				// no null construction
 	Polymorphism(slim_polymorphismid_t p_polymorphism_id, const Mutation *p_mutation_ptr, slim_refcount_t p_prevalence);
 	
+	void Print_ID_Tag(std::ostream &p_out) const;			// includes polymorphism_id_ and tag
 	void Print_ID(std::ostream &p_out) const;			// includes polymorphism_id_ at the beginning
+	void Print_NoID_Tag(std::ostream &p_out) const;	// does not include polymorphism_id_, but includes tag
 	void Print_NoID(std::ostream &p_out) const;	// does not include polymorphism_id_
 	
 	friend bool operator<(const Polymorphism& p_l, const Polymorphism& p_r)
