@@ -3,7 +3,7 @@
 //  SLiM
 //
 //  Created by Ben Haller on 12/13/14.
-//  Copyright (c) 2014-2024 Philipp Messer.  All rights reserved.
+//  Copyright (c) 2014-2025 Philipp Messer.  All rights reserved.
 //	A product of the Messer Lab, http://messerlab.org/slim/
 //
 
@@ -44,7 +44,7 @@ class Community;
 class Species;
 class Subpopulation;
 class Individual;
-class Genome;
+class Haplosome;
 
 
 #pragma mark -
@@ -63,17 +63,17 @@ public:
 	SLiM_DeferredReproductionType type_;
 	Individual *parent1_;
 	Individual *parent2_;
-	Genome *child_genome_1_;
-	Genome *child_genome_2_;
+	Haplosome *child_haplosome_1_;
+	Haplosome *child_haplosome_2_;
 	IndividualSex child_sex_;
 	
 	SLiM_DeferredReproduction_NonRecombinant(SLiM_DeferredReproductionType p_type,
 							  Individual *p_parent1,
 							  Individual *p_parent2,
-							  Genome *p_child_genome_1,
-							  Genome *p_child_genome_2,
+							  Haplosome *p_child_haplosome_1,
+							  Haplosome *p_child_haplosome_2,
 							  IndividualSex p_child_sex) :
-		type_(p_type), parent1_(p_parent1), parent2_(p_parent2), child_genome_1_(p_child_genome_1), child_genome_2_(p_child_genome_2), child_sex_(p_child_sex)
+		type_(p_type), parent1_(p_parent1), parent2_(p_parent2), child_haplosome_1_(p_child_haplosome_1), child_haplosome_2_(p_child_haplosome_2), child_sex_(p_child_sex)
 	{};
 };
 
@@ -81,20 +81,18 @@ class SLiM_DeferredReproduction_Recombinant {
 public:
 	SLiM_DeferredReproductionType type_;
 	Subpopulation *mutorigin_subpop_;
-	Genome *child_genome_;
-	Genome *strand1_;
-	Genome *strand2_;
+	Haplosome *child_haplosome_;
+	Haplosome *strand1_;
+	Haplosome *strand2_;
 	std::vector<slim_position_t> break_vec_;
-	IndividualSex sex_;
 	
 	SLiM_DeferredReproduction_Recombinant(SLiM_DeferredReproductionType p_type,
 							  Subpopulation *p_mutorigin_subpop,
-							  Genome *p_strand1,
-							  Genome *p_strand2,
+							  Haplosome *p_strand1,
+							  Haplosome *p_strand2,
 							  std::vector<slim_position_t> &p_break_vec,
-							  Genome *p_child_genome,
-							  IndividualSex p_sex) :
-		type_(p_type), mutorigin_subpop_(p_mutorigin_subpop), child_genome_(p_child_genome), strand1_(p_strand1), strand2_(p_strand2), sex_(p_sex)
+							  Haplosome *p_child_haplosome) :
+		type_(p_type), mutorigin_subpop_(p_mutorigin_subpop), child_haplosome_(p_child_haplosome), strand1_(p_strand1), strand2_(p_strand2)
 	{
 		std::swap(break_vec_, p_break_vec);		// take ownership of the passed vector with std::swap(), to avoid copying
 	};
@@ -125,9 +123,9 @@ class Population
 	MutationRun mutation_registry_;							// OWNED POINTERS: a registry of all mutations that have been added to this population
 	bool registry_needs_consistency_check_ = false;			// set this to run CheckMutationRegistry() at the end of the cycle
 	
-	// Cache info for TallyMutationReferences...(); see those functions
+	// Cache info for TallyMutationReferences...(), along with Chromosome::cached_tally_haplosome_count_; see those functions
+	bool cached_tallies_valid_ = false;
 	std::vector<Subpopulation*> last_tallied_subpops_;		// NOT OWNED POINTERS
-	slim_refcount_t cached_tally_genome_count_ = 0;			// a value of 0 indicates that the cache is invalid
 	
 public:
 	
@@ -137,24 +135,22 @@ public:
 	Community &community_;
 	Species &species_;
 	
-	// Object pools for individuals and genomes, kept population-wide
-	EidosObjectPool species_genome_pool_;					// a pool out of which genomes are allocated, for within-species locality of memory usage across genomes
-	EidosObjectPool species_individual_pool_;				// a pool out of which individuals are allocated, for within-species locality of memory usage across individuals
-	std::vector<Genome *> species_genome_junkyard_nonnull;	// non-null genomes get put here when we're done with them, so we can reuse them without dealloc/realloc of their mutrun buffers
-	std::vector<Genome *> species_genome_junkyard_null;		// null genomes get put here when we're done with them, so we can reuse them without dealloc/realloc of their mutrun buffers
+	// Object pools for individuals and haplosomes, kept species-wide
+	EidosObjectPool &species_haplosome_pool_;					// NOT OWNED; a pool out of which haplosomes are allocated, for within-species locality
+	EidosObjectPool &species_individual_pool_;					// NOT OWNED; a pool out of which individuals are allocated, for within-species locality
+	std::vector<Individual *> species_individuals_junkyard_;	// individuals get put here when we're done with them, for fast reuse
 	
 #ifdef SLIM_KEEP_MUTTYPE_REGISTRIES
 	bool keeping_muttype_registries_ = false;				// if true, at least one MutationType is also keeping its own registry
 	bool any_muttype_call_count_used_ = false;				// if true, a muttype's muttype_registry_call_count_ has been incremented
 #endif
 	
-	slim_refcount_t total_genome_count_ = 0;				// the number of non-null genomes in the population; a fixed mutation has this count
-#ifdef SLIMGUI
-	slim_refcount_t gui_total_genome_count_ = 0;			// the number of non-null genomes in the selected subpopulations in SLiMgui
-#endif
-	
+#if DEFER_BROKEN
+	// The "defer" flag is simply disregarded at the moment; its design has rotted away,
+	// and needs to be remade anew once things have settled down.
 	std::vector<SLiM_DeferredReproduction_NonRecombinant> deferred_reproduction_nonrecombinant_;
 	std::vector<SLiM_DeferredReproduction_Recombinant> deferred_reproduction_recombinant_;
+#endif
 	
 	std::vector<Substitution*> substitutions_;				// OWNED POINTERS: Substitution objects for all fixed mutations
 	std::unordered_multimap<slim_position_t, Substitution*> treeseq_substitutions_map_;	// TREE SEQUENCE RECORDING; keeps all fixed mutations, hashed by position
@@ -228,18 +224,32 @@ public:
 	bool ApplyModifyChildCallbacks(Individual *p_child, Individual *p_parent1, Individual *p_parent2, bool p_is_selfing, bool p_is_cloning, Subpopulation *p_target_subpop, Subpopulation *p_source_subpop, std::vector<SLiMEidosBlock*> &p_modify_child_callbacks);
 	
 	// apply recombination() callbacks to a generated child; a return of true means the breakpoints were changed
-	bool ApplyRecombinationCallbacks(slim_popsize_t p_parent_index, Genome *p_genome1, Genome *p_genome2, Subpopulation *p_source_subpop, std::vector<slim_position_t> &p_crossovers, std::vector<SLiMEidosBlock*> &p_recombination_callbacks);
+	// note that in the general case (e.g., addRecombinant()), the haplosomes might not belong to the parent
+	bool ApplyRecombinationCallbacks(Individual *p_parent, Haplosome *p_haplosome1, Haplosome *p_haplosome2, std::vector<slim_position_t> &p_crossovers, std::vector<SLiMEidosBlock*> &p_recombination_callbacks);
 	
-	// generate a child genome from parental genomes, with recombination, gene conversion, and mutation
-	void DoCrossoverMutation(Subpopulation *p_source_subpop, Genome &p_child_genome, slim_popsize_t p_parent_index, IndividualSex p_child_sex, IndividualSex p_parent_sex, std::vector<SLiMEidosBlock*> *p_recombination_callbacks, std::vector<SLiMEidosBlock*> *p_mutation_callbacks);
-	void DoHeteroduplexRepair(std::vector<slim_position_t> &p_heteroduplex, std::vector<slim_position_t> &p_breakpoints, Genome *p_parent_genome_1, Genome *p_parent_genome_2, Genome *p_child_genome);
+	// generate a child haplosome from parental haplosome(s), very directly -- no null haplosomes etc., just cross/clone/recombine
+	// these methods are templated with variants for speed; see also MungeIndividualCrossed() etc.
+	template <const bool f_treeseq, const bool f_callbacks>
+	void HaplosomeCrossed(Chromosome &p_chromosome, Haplosome &p_child_haplosome, Haplosome *parent_haplosome_1, Haplosome *parent_haplosome_2, std::vector<SLiMEidosBlock*> *p_recombination_callbacks, std::vector<SLiMEidosBlock*> *p_mutation_callbacks);
 	
-	// generate a child genome from parental genomes, with predetermined recombination and mutation
-	void DoRecombinantMutation(Subpopulation *p_mutorigin_subpop, Genome &p_child_genome, Genome *p_parent_genome_1, Genome *p_parent_genome_2, IndividualSex p_parent_sex, std::vector<slim_position_t> &p_breakpoints, std::vector<SLiMEidosBlock*> *p_mutation_callbacks);
+	template <const bool f_treeseq, const bool f_callbacks>
+	void HaplosomeCloned(Chromosome &p_chromosome, Haplosome &p_child_haplosome, Haplosome *parent_haplosome, std::vector<SLiMEidosBlock*> *p_mutation_callbacks);
 	
-	// generate a child genome from a single parental genome, without recombination or gene conversion, but with mutation
-	void DoClonalMutation(Subpopulation *p_mutorigin_subpop, Genome &p_child_genome, Genome &p_parent_genome, IndividualSex p_child_sex, std::vector<SLiMEidosBlock*> *p_mutation_callbacks);
+	template <const bool f_treeseq, const bool f_callbacks>
+	void HaplosomeRecombined(Chromosome &p_chromosome, Haplosome &p_child_haplosome, Haplosome *parent_haplosome_1, Haplosome *parent_haplosome_2, std::vector<slim_position_t> &p_breakpoints, std::vector<SLiMEidosBlock*> *p_mutation_callbacks);
 	
+	void (Population::*HaplosomeCrossed_TEMPLATED)(Chromosome &p_chromosome, Haplosome &p_child_haplosome, Haplosome *parent_haplosome_1, Haplosome *parent_haplosome_2, std::vector<SLiMEidosBlock*> *p_recombination_callbacks, std::vector<SLiMEidosBlock*> *p_mutation_callbacks) = nullptr;
+	void (Population::*HaplosomeCloned_TEMPLATED)(Chromosome &p_chromosome, Haplosome &p_child_haplosome, Haplosome *parent_haplosome, std::vector<SLiMEidosBlock*> *p_mutation_callbacks) = nullptr;
+	void (Population::*HaplosomeRecombined_TEMPLATED)(Chromosome &p_chromosome, Haplosome &p_child_haplosome, Haplosome *parent_haplosome_1, Haplosome *parent_haplosome_2, std::vector<slim_position_t> &p_breakpoints, std::vector<SLiMEidosBlock*> *p_mutation_callbacks) = nullptr;
+	
+	void DoHeteroduplexRepair(std::vector<slim_position_t> &p_heteroduplex, slim_position_t *p_breakpoints, int p_breakpoints_count, Haplosome *p_parent_haplosome_1, Haplosome *p_parent_haplosome_2, Haplosome *p_child_haplosome);
+	
+	// generate offspring within a reproduction() callback using templated Subpopulation methods; these pointers get
+	// set up at the beginning of each tick's reproduction() callback stage, and should not be used outside of it
+	Individual *(Subpopulation::*GenerateIndividualCrossed_TEMPLATED)(Individual *p_parent1, Individual *p_parent2, IndividualSex p_child_sex) = nullptr;
+	Individual *(Subpopulation::*GenerateIndividualSelfed_TEMPLATED)(Individual *p_parent) = nullptr;
+	Individual *(Subpopulation::*GenerateIndividualCloned_TEMPLATED)(Individual *p_parent) = nullptr;
+
 	// An internal method that validates cached fitness values kept by Mutation objects
 	void ValidateMutationFitnessCaches(void);
 	
@@ -249,9 +259,9 @@ public:
 	// Scan through all mutation runs in the simulation and unique them
 	void UniqueMutationRuns(void);
 	
-	// Scan through all genomes and either split or join their mutation runs, to double or halve the number of runs per genome
-	void SplitMutationRuns(int32_t p_new_mutrun_count);
-	void JoinMutationRuns(int32_t p_new_mutrun_count);
+	// Scan through all haplosomes and either split or join their mutation runs, to double or halve the number of runs per haplosome
+	void SplitMutationRunsForChromosome(int32_t p_new_mutrun_count, Chromosome *p_chromosome);
+	void JoinMutationRunsForChromosome(int32_t p_new_mutrun_count, Chromosome *p_chromosome);
 	
 	// Tally mutations and remove fixed/lost mutations
 	void MaintainMutationRegistry(void);
@@ -259,48 +269,52 @@ public:
 	// Tally MutationRun usage and free unused MutationRuns.  Note that all of these tallying methods tally into
 	// the same use_count_ counter kept by MutationRun, so a new tally wipes the results of the previous tally.
 	// Also note that the mutation tallying methods below call these methods to tally mutation runs first, so
-	// the mutation run tallies will be altered as a side effect of doing a mutation tally.  The return value
-	// for all of these methods is the number of non-null genomes that were tallied across.
-	slim_refcount_t TallyMutationRunReferencesForPopulation(void);
-	slim_refcount_t TallyMutationRunReferencesForSubpops(std::vector<Subpopulation*> *p_subpops_to_tally);
-	slim_refcount_t TallyMutationRunReferencesForGenomes(const Genome * const *genomes_ptr, slim_popsize_t genomes_count);
+	// the mutation run tallies will be altered as a side effect of doing a mutation tally.  These methods all
+	// place the number of non-null haplosomes that were tallied across into the tallied_haplosome_count_ value
+	// of each chromosome involved in the tally.
+	void TallyMutationRunReferencesForPopulationForChromosome(Chromosome *p_chromosome);
+	void TallyMutationRunReferencesForPopulation(bool p_clock_for_mutrun_experiments);
+	void TallyMutationRunReferencesForSubpopsForChromosome(std::vector<Subpopulation*> *p_subpops_to_tally, Chromosome *p_chromosome);
+	void TallyMutationRunReferencesForSubpops(std::vector<Subpopulation*> *p_subpops_to_tally);
+	void TallyMutationRunReferencesForHaplosomes(const Haplosome * const *haplosomes_ptr, slim_popsize_t haplosomes_count);
 	void FreeUnusedMutationRuns(void);	// depends upon a previous tally by TallyMutationRunReferencesForPopulation()!
 	
 	// Tally Mutation usage; these count the total number of times that each Mutation in the registry is referenced
-	// by a population (or a set of subpopulations, or a set of genomes), putting the usage counts into the refcount
+	// by a population (or a set of subpopulations, or a set of haplosomes), putting the usage counts into the refcount
 	// block kept by Mutation.  For the whole population, and for a set of subpops, cache info is maintained so the
-	// tally can be reused when possible.  For a set of genomes, the result is not cached, and so the cache is
-	// always invalidated.  The maximum number of references (the total number of non-null genomes tallied) is
-	// always returned.  When tallying across all subpopulations, total_genome_count_ is also set to this same
+	// tally can be reused when possible.  For a set of haplosomes, the result is not cached, and so the cache is
+	// always invalidated.  The maximum number of references (the total number of non-null haplosomes tallied) is
+	// always placed into the tallied_haplosome_count_ value for each chromosome involved in the tally.  When
+	// tallying across all subpopulations, total_haplosome_count_ for each chromosome is also set to this same
 	// value, which is the maximum possible number of references (i.e. fixation), as a side effect.  The cache
 	// of tallies can be invalidated by calling InvalidateMutationReferencesCache().
-	inline void InvalidateMutationReferencesCache(void) { last_tallied_subpops_.clear(); cached_tally_genome_count_ = 0; }
-	
-	slim_refcount_t TallyMutationReferencesAcrossPopulation(bool p_force_recache);
-	slim_refcount_t TallyMutationReferencesAcrossSubpopulations(std::vector<Subpopulation*> *p_subpops_to_tally, bool p_force_recache);
-	slim_refcount_t TallyMutationReferencesAcrossGenomes(const Genome * const *genomes, slim_popsize_t genomes_count);
-	
-	slim_refcount_t _CountNonNullGenomes(void);
+	void InvalidateMutationReferencesCache(void);
+
+	void TallyMutationReferencesAcrossPopulation(bool p_clock_for_mutrun_experiments);
 #ifdef SLIMGUI
-	void _CopyRefcountsToSLiMgui(void);
+	void TallyMutationReferencesAcrossPopulation_SLiMgui(void);		// tallies selected subpops into SLiMgui-private counters
 #endif
-	void _TallyMutationReferences_FAST_FromMutationRunUsage(void);
+	void TallyMutationReferencesAcrossSubpopulations(std::vector<Subpopulation*> *p_subpops_to_tally);
+	void TallyMutationReferencesAcrossHaplosomes(const Haplosome * const *haplosomes, slim_popsize_t haplosomes_count);
+	
+	slim_refcount_t _CountNonNullHaplosomesForChromosome(Chromosome *p_chromosome);
+	void _TallyMutationReferences_FAST_FromMutationRunUsage(bool p_clock_for_mutrun_experiments);
 #if DEBUG
-	void _CheckMutationTallyAcrossGenomes(const Genome * const *genomes_ptr, slim_popsize_t genomes_count);
+	void _CheckMutationTallyAcrossHaplosomes(const Haplosome * const *haplosomes_ptr, slim_popsize_t haplosomes_count, std::string caller_name);
 #endif
 	
 	// Eidos back-end code that counts up tallied mutations, to be called after TallyMutationReferences...().
 	// These methods correctly handle cases where the mutations are fixed, removed, substituted, lost, etc.,
 	// to return the correct frequency/count values to the user as an EidosValue_SP.
-	EidosValue_SP Eidos_FrequenciesForTalliedMutations(EidosValue *mutations_value, int total_genome_count);
-	EidosValue_SP Eidos_CountsForTalliedMutations(EidosValue *mutations_value, int total_genome_count);
+	EidosValue_SP Eidos_FrequenciesForTalliedMutations(EidosValue *mutations_value);
+	EidosValue_SP Eidos_CountsForTalliedMutations(EidosValue *mutations_value);
 	
 	// Handle negative fixation (remove from the registry) and positive fixation (convert to Substitution).
 	// This uses reference counts from TallyMutationReferencesAcrossPopulation(), which must be called before this method.
 	void RemoveAllFixedMutations(void);
 	
 	// check the registry for any bad entries (i.e. zombies, mutations with an incorrect state_)
-	void CheckMutationRegistry(bool p_check_genomes);
+	void CheckMutationRegistry(bool p_check_haplosomes);
 	inline void SetMutationRegistryNeedsCheck(void) { registry_needs_consistency_check_ = true; }
 	inline bool MutationRegistryNeedsCheck(void) { return registry_needs_consistency_check_; }
 	
@@ -327,8 +341,10 @@ public:
 	// step forward a generation: make the children become the parents
 	void SwapGenerations(void);
 	
-	// Clear all parental genomes to use nullptr for their mutation runs, so they are ready to reuse in the next tick
-	void ClearParentalGenomes(void);
+#if SLIM_CLEAR_HAPLOSOMES
+	// Clear all parental haplosomes to use nullptr for their mutation runs, so they are ready to reuse in the next tick
+	void ClearParentalHaplosomes(void);
+#endif
 	
 	//********** nonWF methods
 
@@ -338,30 +354,39 @@ public:
 	// move individuals as requested by survival() callbacks
 	void ResolveSurvivalPhaseMovement(void);
 	
-	// checks for deferred genomes in queue right now; allows optimization when none are present
-	inline bool HasDeferredGenomes(void) { return ((deferred_reproduction_nonrecombinant_.size() > 0) || (deferred_reproduction_recombinant_.size() > 0)); }
-	void CheckForDeferralInGenomesVector(Genome **p_genomes, size_t p_elements_size, const std::string &p_caller);
-	void CheckForDeferralInGenomes(EidosValue_Object *p_genomes, const std::string &p_caller);
-	void CheckForDeferralInIndividualsVector(Individual **p_individuals, size_t p_elements_size, const std::string &p_caller);
+	// checks for deferred haplosomes in queue right now; allows optimization when none are present
+#if DEFER_BROKEN
+	inline bool HasDeferredHaplosomes(void) { return ((deferred_reproduction_nonrecombinant_.size() > 0) || (deferred_reproduction_recombinant_.size() > 0)); }
+	void CheckForDeferralInHaplosomesVector(Haplosome **p_haplosomes, size_t p_elements_size, const std::string &p_caller);
+	void CheckForDeferralInHaplosomes(EidosValue_Object *p_haplosomes, const std::string &p_caller);
+	void CheckForDeferralInIndividualsVector(Individual * const *p_individuals, size_t p_elements_size, const std::string &p_caller);
+#else
+	inline bool HasDeferredHaplosomes(void) { return false; }
+	inline void CheckForDeferralInHaplosomesVector(__attribute__ ((unused)) Haplosome **p_haplosomes, __attribute__ ((unused)) size_t p_elements_size, __attribute__ ((unused)) const std::string &p_caller) {}
+	inline void CheckForDeferralInHaplosomes(__attribute__ ((unused)) EidosValue_Object *p_haplosomes, __attribute__ ((unused)) const std::string &p_caller) {}
+	inline void CheckForDeferralInIndividualsVector(__attribute__ ((unused)) Individual * const *p_individuals, __attribute__ ((unused)) size_t p_elements_size, __attribute__ ((unused)) const std::string &p_caller) {}
+#endif
 	
+#if DEFER_BROKEN
 	void DoDeferredReproduction(void);
+#endif
 	
 	//********** methods for all models
 	
 	void PurgeRemovedSubpopulations(void);
 
-	// print all mutations and all genomes to a stream
-	void PrintAll(std::ostream &p_out, bool p_output_spatial_positions, bool p_output_ages, bool p_output_ancestral_nucs, bool p_output_pedigree_ids) const;
-	void PrintAllBinary(std::ostream &p_out, bool p_output_spatial_positions, bool p_output_ages, bool p_output_ancestral_nucs, bool p_output_pedigree_ids) const;
+	// print all mutations and all haplosomes to a stream
+	// note that text output is now at Individual::PrintIndividuals_SLiM()
+	void PrintAllBinary(std::ostream &p_out, bool p_output_spatial_positions, bool p_output_ages, bool p_output_ancestral_nucs, bool p_output_pedigree_ids, bool p_output_object_tags, bool p_output_substitutions) const;
 	
-	// print sample of p_sample_size genomes from subpopulation p_subpop_id, using SLiM's own format
-	void PrintSample_SLiM(std::ostream &p_out, Subpopulation &p_subpop, slim_popsize_t p_sample_size, bool p_replace, IndividualSex p_requested_sex) const;
+	// print sample of p_sample_size haplosomes from subpopulation p_subpop_id, using SLiM's own format
+	void PrintSample_SLiM(std::ostream &p_out, Subpopulation &p_subpop, slim_popsize_t p_sample_size, bool p_replace, IndividualSex p_requested_sex, const Chromosome &p_chromosome) const;
 	
-	// print sample of p_sample_size genomes from subpopulation p_subpop_id, using "ms" format
+	// print sample of p_sample_size haplosomes from subpopulation p_subpop_id, using "ms" format
 	void PrintSample_MS(std::ostream &p_out, Subpopulation &p_subpop, slim_popsize_t p_sample_size, bool p_replace, IndividualSex p_requested_sex, const Chromosome &p_chromosome, bool p_filter_monomorphic) const;
 	
-	// print sample of p_sample_size genomes from subpopulation p_subpop_id, using "vcf" format
-	void PrintSample_VCF(std::ostream &p_out, Subpopulation &p_subpop, slim_popsize_t p_sample_size, bool p_replace, IndividualSex p_requested_sex, bool p_output_multiallelics, bool p_simplify_nucs, bool p_output_nonnucs) const;
+	// print sample of p_sample_size haplosomes from subpopulation p_subpop_id, using "vcf" format
+	void PrintSample_VCF(std::ostream &p_out, Subpopulation &p_subpop, slim_popsize_t p_sample_size, bool p_replace, IndividualSex p_requested_sex, const Chromosome &p_chromosome, bool p_output_multiallelics, bool p_simplify_nucs, bool p_output_nonnucs, bool p_group_as_individuals) const;
 	
 	// remove subpopulations, purge all mutations and substitutions, etc.; called before InitializePopulationFrom[Text|Binary]File()
 	void RemoveAllSubpopulationInfo(void);
