@@ -3,7 +3,7 @@
 //  SLiM
 //
 //  Created by Ben Haller on 12/13/14.
-//  Copyright (c) 2014-2024 Philipp Messer.  All rights reserved.
+//  Copyright (c) 2014-2025 Philipp Messer.  All rights reserved.
 //	A product of the Messer Lab, http://messerlab.org/slim/
 //
 
@@ -22,6 +22,7 @@
 #include "slim_globals.h"
 #include "eidos_call_signature.h"
 #include "eidos_property_signature.h"
+#include "species.h"
 
 #include <iostream>
 #include <algorithm>
@@ -34,21 +35,36 @@
 #pragma mark -
 
 Substitution::Substitution(Mutation &p_mutation, slim_tick_t p_fixation_tick) :
-	EidosDictionaryRetained(), mutation_type_ptr_(p_mutation.mutation_type_ptr_), position_(p_mutation.position_), selection_coeff_(p_mutation.selection_coeff_), subpop_index_(p_mutation.subpop_index_), origin_tick_(p_mutation.origin_tick_), fixation_tick_(p_fixation_tick), nucleotide_(p_mutation.nucleotide_), mutation_id_(p_mutation.mutation_id_), tag_value_(p_mutation.tag_value_)
+	EidosDictionaryRetained(), mutation_type_ptr_(p_mutation.mutation_type_ptr_), position_(p_mutation.position_), selection_coeff_(p_mutation.selection_coeff_), subpop_index_(p_mutation.subpop_index_), origin_tick_(p_mutation.origin_tick_), fixation_tick_(p_fixation_tick), chromosome_index_(p_mutation.chromosome_index_), nucleotide_(p_mutation.nucleotide_), mutation_id_(p_mutation.mutation_id_), tag_value_(p_mutation.tag_value_)
 	
 {
 	AddKeysAndValuesFrom(&p_mutation);
 	// No call to ContentsChanged() here; we know we use Dictionary not DataFrame, and Mutation already vetted the dictionary
 }
 
-Substitution::Substitution(slim_mutationid_t p_mutation_id, MutationType *p_mutation_type_ptr, slim_position_t p_position, double p_selection_coeff, slim_objectid_t p_subpop_index, slim_tick_t p_tick, slim_tick_t p_fixation_tick, int8_t p_nucleotide) :
-mutation_type_ptr_(p_mutation_type_ptr), position_(p_position), selection_coeff_(static_cast<slim_selcoeff_t>(p_selection_coeff)), subpop_index_(p_subpop_index), origin_tick_(p_tick), fixation_tick_(p_fixation_tick), nucleotide_(p_nucleotide), mutation_id_(p_mutation_id), tag_value_(SLIM_TAG_UNSET_VALUE)
+Substitution::Substitution(slim_mutationid_t p_mutation_id, MutationType *p_mutation_type_ptr, slim_chromosome_index_t p_chromosome_index, slim_position_t p_position, double p_selection_coeff, slim_objectid_t p_subpop_index, slim_tick_t p_tick, slim_tick_t p_fixation_tick, int8_t p_nucleotide) :
+mutation_type_ptr_(p_mutation_type_ptr), position_(p_position), selection_coeff_(static_cast<slim_selcoeff_t>(p_selection_coeff)), subpop_index_(p_subpop_index), origin_tick_(p_tick), fixation_tick_(p_fixation_tick), chromosome_index_(p_chromosome_index), nucleotide_(p_nucleotide), mutation_id_(p_mutation_id), tag_value_(SLIM_TAG_UNSET_VALUE)
 {
 }
 
 void Substitution::PrintForSLiMOutput(std::ostream &p_out) const
 { 
-	p_out << mutation_id_ << " m" << mutation_type_ptr_->mutation_type_id_ << " " << position_ << " " << selection_coeff_ << " " << mutation_type_ptr_->dominance_coeff_ << " p" << subpop_index_ << " " << origin_tick_ << " "<< fixation_tick_;
+	p_out << mutation_id_ << " m" << mutation_type_ptr_->mutation_type_id_ << " " << position_;
+	
+	// BCH 2/2/2025: Note that in multi-chrom models, this method now prints the chromosome symbol after the position
+	// For brevity and backward compatibility, the chromosome symbol is not printed in single-chromosome models
+	Species &species = mutation_type_ptr_->species_;
+	const std::vector<Chromosome *> &chromosomes = species.Chromosomes();
+	
+	if (chromosomes.size() > 1)
+	{
+		Chromosome *chromosome = chromosomes[chromosome_index_];
+		
+		p_out << " \"" << chromosome->Symbol() << "\"";
+	}
+	
+	// and then the remainder of the output line
+	p_out << " " << selection_coeff_ << " " << mutation_type_ptr_->dominance_coeff_ << " p" << subpop_index_ << " " << origin_tick_ << " "<< fixation_tick_;
 	
 	// output a nucleotide if available
 	if (mutation_type_ptr_->nucleotide_based_)
@@ -57,6 +73,40 @@ void Substitution::PrintForSLiMOutput(std::ostream &p_out) const
 	p_out << std::endl;
 }
 
+void Substitution::PrintForSLiMOutput_Tag(std::ostream &p_out) const
+{ 
+	// BCH 4/7/2025: This is a copy of PrintForSLiMOutput() with output of tag_value_ added at the end
+	
+	p_out << mutation_id_ << " m" << mutation_type_ptr_->mutation_type_id_ << " " << position_;
+	
+	// BCH 2/2/2025: Note that in multi-chrom models, this method now prints the chromosome symbol after the position
+	// For brevity and backward compatibility, the chromosome symbol is not printed in single-chromosome models
+	Species &species = mutation_type_ptr_->species_;
+	const std::vector<Chromosome *> &chromosomes = species.Chromosomes();
+	
+	if (chromosomes.size() > 1)
+	{
+		Chromosome *chromosome = chromosomes[chromosome_index_];
+		
+		p_out << " \"" << chromosome->Symbol() << "\"";
+	}
+	
+	// and then the remainder of the output line
+	p_out << " " << selection_coeff_ << " " << mutation_type_ptr_->dominance_coeff_ << " p" << subpop_index_ << " " << origin_tick_ << " "<< fixation_tick_;
+	
+	// output a nucleotide if available
+	if (mutation_type_ptr_->nucleotide_based_)
+		p_out << " " << gSLiM_Nucleotides[nucleotide_];
+	
+	// output the tag value, or '?' or the tag is not defined
+	if (tag_value_ == SLIM_TAG_UNSET_VALUE)
+		p_out << ' ' << '?';
+	else
+		p_out << ' ' << tag_value_;
+	
+	p_out << std::endl;
+}
+
 
 //
 //	Eidos support
@@ -72,7 +122,7 @@ const EidosClass *Substitution::Class(void) const
 
 void Substitution::Print(std::ostream &p_ostream) const
 {
-	p_ostream << Class()->ClassName() << "<" << mutation_id_ << ":" << selection_coeff_ << ">";
+	p_ostream << Class()->ClassNameForDisplay() << "<" << mutation_id_ << ":" << selection_coeff_ << ">";
 }
 
 EidosValue_SP Substitution::GetProperty(EidosGlobalStringID p_property_id)
@@ -81,6 +131,14 @@ EidosValue_SP Substitution::GetProperty(EidosGlobalStringID p_property_id)
 	switch (p_property_id)
 	{
 			// constants
+		case gID_chromosome:
+		{
+			Species &species = mutation_type_ptr_->species_;
+			const std::vector<Chromosome *> &chromosomes = species.Chromosomes();
+			Chromosome *chromosome = chromosomes[chromosome_index_];
+			
+			return EidosValue_SP(new (gEidosValuePool->AllocateChunk()) EidosValue_Object(chromosome, gSLiM_Chromosome_Class));
+		}
 		case gID_id:					// ACCELERATED
 			return EidosValue_SP(new (gEidosValuePool->AllocateChunk()) EidosValue_Int(mutation_id_));
 		case gID_mutationType:			// ACCELERATED
@@ -384,6 +442,7 @@ const std::vector<EidosPropertySignature_CSP> *Substitution_Class::Properties(vo
 		
 		properties = new std::vector<EidosPropertySignature_CSP>(*super::Properties());
 		
+		properties->emplace_back((EidosPropertySignature *)(new EidosPropertySignature(gStr_chromosome,			true,	kEidosValueMaskObject | kEidosValueMaskSingleton, gSLiM_Chromosome_Class)));
 		properties->emplace_back((EidosPropertySignature *)(new EidosPropertySignature(gStr_id,					true,	kEidosValueMaskInt | kEidosValueMaskSingleton))->DeclareAcceleratedGet(Substitution::GetProperty_Accelerated_id));
 		properties->emplace_back((EidosPropertySignature *)(new EidosPropertySignature(gStr_mutationType,		true,	kEidosValueMaskObject | kEidosValueMaskSingleton, gSLiM_MutationType_Class))->DeclareAcceleratedGet(Substitution::GetProperty_Accelerated_mutationType));
 		properties->emplace_back((EidosPropertySignature *)(new EidosPropertySignature(gStr_position,			true,	kEidosValueMaskInt | kEidosValueMaskSingleton))->DeclareAcceleratedGet(Substitution::GetProperty_Accelerated_position));
