@@ -3,7 +3,7 @@
 //  Eidos
 //
 //  Created by Ben Haller on 8/3/15.
-//  Copyright (c) 2015-2024 Philipp Messer.  All rights reserved.
+//  Copyright (c) 2015-2025 Philipp Messer.  All rights reserved.
 //	A product of the Messer Lab, http://messerlab.org/slim/
 //
 
@@ -97,7 +97,7 @@ void EidosPropertySignature::CheckAssignedValue(const EidosValue &p_value) const
 			// This check is applied only if the value contains elements, since an empty object does not know its type.
 			if (value_type_ok && value_class_ && (((EidosValue_Object *)&p_value)->Class() != value_class_) && (p_value.Count() > 0))
 			{
-				EIDOS_TERMINATION << "ERROR (EidosPropertySignature::CheckAssignedValue): object value cannot be object element type " << p_value.ElementType() << " for " << PropertyType() << " property " << property_name_ << "; expected object element type " << value_class_->ClassName() << "." << EidosTerminate(nullptr);
+				EIDOS_TERMINATION << "ERROR (EidosPropertySignature::CheckAssignedValue): object value cannot be object element type " << p_value.ElementType() << " for " << PropertyType() << " property " << property_name_ << "; expected object element type " << value_class_->ClassNameForDisplay() << "." << EidosTerminate(nullptr);
 			}
 			break;
 	}
@@ -150,7 +150,7 @@ void EidosPropertySignature::CheckResultValue(const EidosValue &p_value) const
 			if (value_type_ok && value_class_ && (((EidosValue_Object *)&p_value)->Class() != value_class_) && (p_value.Count() > 0))
 			{
 				if (!((EidosValue_Object *)&p_value)->Class()->IsSubclassOfClass(value_class_))
-					EIDOS_TERMINATION << "ERROR (EidosPropertySignature::CheckResultValue): (internal error) object value cannot be object element type " << p_value.ElementType() << " for " << PropertyType() << " property " << property_name_ << "; expected object element type " << value_class_->ClassName() << "." << EidosTerminate(nullptr);
+					EIDOS_TERMINATION << "ERROR (EidosPropertySignature::CheckResultValue): (internal error) object value cannot be object element type " << p_value.ElementType() << " for " << PropertyType() << " property " << property_name_ << "; expected object element type " << value_class_->ClassNameForDisplay() << "." << EidosTerminate(nullptr);
 			}
 			break;
 	}
@@ -192,7 +192,7 @@ void EidosPropertySignature::CheckAggregateResultValue(const EidosValue &p_value
 			if (value_type_ok && value_class_ && (((EidosValue_Object *)&p_value)->Class() != value_class_) && (p_value.Count() > 0))
 			{
 				if (!((EidosValue_Object *)&p_value)->Class()->IsSubclassOfClass(value_class_))
-					EIDOS_TERMINATION << "ERROR (EidosPropertySignature::CheckAggregateResultValue): (internal error) object value cannot be object element type " << p_value.ElementType() << " for " << PropertyType() << " property " << property_name_ << "; expected object element type " << value_class_->ClassName() << "." << EidosTerminate(nullptr);
+					EIDOS_TERMINATION << "ERROR (EidosPropertySignature::CheckAggregateResultValue): (internal error) object value cannot be object element type " << p_value.ElementType() << " for " << PropertyType() << " property " << property_name_ << "; expected object element type " << value_class_->ClassNameForDisplay() << "." << EidosTerminate(nullptr);
 			}
 			break;
 	}
@@ -265,6 +265,7 @@ EidosPropertySignature *EidosPropertySignature::DeclareAcceleratedSet(Eidos_Acce
 
 EidosPropertySignature *EidosPropertySignature::MarkDeprecated(void)
 {
+	// At present, the only consequence of deprecation is that the property/method is not listed in the documentation
 	deprecated_ = true;
 	return this;
 }
