@@ -3,7 +3,7 @@
 //  Eidos
 //
 //  Created by Ben Haller on 4/7/15.
-//  Copyright (c) 2015-2024 Philipp Messer.  All rights reserved.
+//  Copyright (c) 2015-2025 Philipp Messer.  All rights reserved.
 //	A product of the Messer Lab, http://messerlab.org/slim/
 //
 
@@ -55,7 +55,7 @@ static int gEidosTestFailureCount = 0;
 void EidosAssertScriptSuccess(const std::string &p_script_string, const EidosValue_SP &p_correct_result)
 {
 	{
-	EidosScript script(p_script_string, -1);
+	EidosScript script(p_script_string);
 	EidosValue_SP result;
 	EidosSymbolTable symbol_table(EidosSymbolTableType::kGlobalVariablesTable, gEidosConstantsSymbolTable);
 	
@@ -70,8 +70,7 @@ void EidosAssertScriptSuccess(const std::string &p_script_string, const EidosVal
 	{
 		std::cerr << p_script_string << " : " << EIDOS_OUTPUT_FAILURE_TAG << " : raise during Tokenize(): " << Eidos_GetTrimmedRaiseMessage() << std::endl;
 		
-		gEidosErrorContext.currentScript = nullptr;
-		gEidosErrorContext.executingRuntimeScript = false;
+		ClearErrorContext();
 		return;
 	}
 	
@@ -82,8 +81,7 @@ void EidosAssertScriptSuccess(const std::string &p_script_string, const EidosVal
 	{
 		std::cerr << p_script_string << " : " << EIDOS_OUTPUT_FAILURE_TAG << " : raise during ParseToAST(): " << Eidos_GetTrimmedRaiseMessage() << std::endl;
 		
-		gEidosErrorContext.currentScript = nullptr;
-		gEidosErrorContext.executingRuntimeScript = false;
+		ClearErrorContext();
 		return;
 	}
 	
@@ -98,9 +96,7 @@ void EidosAssertScriptSuccess(const std::string &p_script_string, const EidosVal
 	{
 		std::cerr << p_script_string << " : " << EIDOS_OUTPUT_FAILURE_TAG << " : raise during EvaluateInterpreterBlock(): " << Eidos_GetTrimmedRaiseMessage() << std::endl;
 		
-		gEidosErrorContext.currentScript = nullptr;
-		gEidosErrorContext.executingRuntimeScript = false;
-		
+		ClearErrorContext();
 		return;
 	}
 	
@@ -139,8 +135,7 @@ void EidosAssertScriptSuccess(const std::string &p_script_string, const EidosVal
 		//std::cerr << p_script_string << " == " << p_correct_result->Type() << "(" << *p_correct_result << ") : " << EIDOS_OUTPUT_SUCCESS_TAG << endl;
 	}
 	
-	gEidosErrorContext.currentScript = nullptr;
-	gEidosErrorContext.executingRuntimeScript = false;
+	ClearErrorContext();
 	
 	if (gEidos_DictionaryNonRetainReleaseReferenceCounter > 0)
 		std::cerr << "WARNING (EidosAssertScriptSuccess): gEidos_DictionaryNonRetainReleaseReferenceCounter == " << gEidos_DictionaryNonRetainReleaseReferenceCounter << " at end of test!" << std::endl;
@@ -204,7 +199,7 @@ void EidosAssertScriptRaise(const std::string &p_script_string, const int p_bad_
 {
 	{
 	std::string reason_snip(p_reason_snip);
-	EidosScript script(p_script_string, -1);
+	EidosScript script(p_script_string);
 	EidosSymbolTable symbol_table(EidosSymbolTableType::kGlobalVariablesTable, gEidosConstantsSymbolTable);
 	EidosFunctionMap function_map(*EidosInterpreter::BuiltInFunctionMap());
 	
@@ -231,8 +226,7 @@ void EidosAssertScriptRaise(const std::string &p_script_string, const int p_bad_
 		if (raise_message.find(reason_snip) != std::string::npos)
 		{
 			if ((gEidosErrorContext.errorPosition.characterStartOfError == -1) ||
-				(gEidosErrorContext.errorPosition.characterEndOfError == -1) ||
-				!gEidosErrorContext.currentScript)
+				(gEidosErrorContext.errorPosition.characterEndOfError == -1))
 			{
 				gEidosTestFailureCount++;
 				
@@ -273,8 +267,7 @@ void EidosAssertScriptRaise(const std::string &p_script_string, const int p_bad_
 		}
 	}
 	
-	gEidosErrorContext.currentScript = nullptr;
-	gEidosErrorContext.executingRuntimeScript = false;
+	ClearErrorContext();
 	
 	if (gEidos_DictionaryNonRetainReleaseReferenceCounter > 0)
 		std::cerr << "WARNING (EidosAssertScriptRaise): gEidos_DictionaryNonRetainReleaseReferenceCounter == " << gEidos_DictionaryNonRetainReleaseReferenceCounter << " at end of test!" << std::endl;
@@ -1162,7 +1155,7 @@ int RunEidosTests(void)
 	
 #if 0
 	// Speed comparison between Poisson and binomial draws in various parameter regimes, using the GSL's own code.
-	// For doing things like drawing the number of recombination or mutation events that happen across a genome,
+	// For doing things like drawing the number of recombination or mutation events that happen across a haplosome,
 	// the binomial distribution is technically correct (# trials, probability per trial), but we have been using
 	// the Poisson distribution because it is an extremely good approximation for the binomial in the regimes we
 	// normally run in (e.g., chromosomes longer than ~100 sites, per-site probabilities less than 1e-3).  And
@@ -1856,19 +1849,19 @@ void _RunFunctionDispatchTests(void)
 {
 	// test function dispatch, default arguments, and named arguments
 	EidosAssertScriptSuccess_I("abs(-10);", 10);
-	EidosAssertScriptRaise("abs();", 0, "missing required argument x");
+	EidosAssertScriptRaise("abs();", 0, "missing required argument 'x'");
 	EidosAssertScriptRaise("abs(-10, -10);", 0, "too many arguments supplied");
 	EidosAssertScriptRaise("abs(x=-10, -10);", 0, "too many arguments supplied");
 	EidosAssertScriptSuccess_I("abs(x=-10);", 10);
 	EidosAssertScriptRaise("abs(y=-10);", 0, "skipped over required argument");
 	EidosAssertScriptRaise("abs(x=-10, x=-10);", 0, "supplied more than once");
-	EidosAssertScriptRaise("abs(x=-10, y=-10);", 0, "unrecognized named argument y");
+	EidosAssertScriptRaise("abs(x=-10, y=-10);", 0, "unrecognized named argument 'y'");
 	EidosAssertScriptRaise("abs(y=-10, x=-10);", 0, "skipped over required argument");
 	
 	EidosAssertScriptSuccess_I("integerDiv(6, 3);", 2);
 	EidosAssertScriptRaise("integerDiv(6, 3, 3);", 0, "too many arguments supplied");
 	EidosAssertScriptRaise("integerDiv(x=6, y=3, 3);", 0, "too many arguments supplied");
-	EidosAssertScriptRaise("integerDiv(6);", 0, "missing required argument y");
+	EidosAssertScriptRaise("integerDiv(6);", 0, "missing required argument 'y'");
 	EidosAssertScriptSuccess_I("integerDiv(x=6, y=3);", 2);
 	EidosAssertScriptRaise("integerDiv(y=6, 3);", 0, "skipped over required argument");
 	EidosAssertScriptRaise("integerDiv(y=6, x=3);", 0, "skipped over required argument");
@@ -1879,26 +1872,28 @@ void _RunFunctionDispatchTests(void)
 	EidosAssertScriptSuccess_IV("seq(1, 3, NULL);", {1, 2, 3});
 	EidosAssertScriptSuccess_IV("seq(1, 3, by=1);", {1, 2, 3});
 	EidosAssertScriptSuccess_IV("seq(1, 3, by=NULL);", {1, 2, 3});
-	EidosAssertScriptRaise("seq(1, 3, x=1);", 0, "ran out of optional arguments");
+	EidosAssertScriptRaise("seq(10, to=20, from=10);", 0, "supplied twice in the argument list");
+	EidosAssertScriptRaise("seq(10, 20, foo=20);", 0, "no parameter with that name");
+	EidosAssertScriptRaise("rainbow(10, v=0.5, s=0.5);", 0, "supplied out of order");
 	EidosAssertScriptRaise("seq(1, 3, by=1, length=1, by=1);", 0, "supplied more than once");
 	EidosAssertScriptRaise("seq(1, 3, length=1, by=1);", 0, "supplied out of order");
 	EidosAssertScriptSuccess_IV("seq(1, 3);", {1, 2, 3});
-	EidosAssertScriptRaise("seq(by=1, 1, 3);", 0, "named argument by skipped over required argument");
-	EidosAssertScriptRaise("seq(by=NULL, 1, 3);", 0, "named argument by skipped over required argument");
+	EidosAssertScriptRaise("seq(by=1, 1, 3);", 0, "named argument 'by' skipped over required argument");
+	EidosAssertScriptRaise("seq(by=NULL, 1, 3);", 0, "named argument 'by' skipped over required argument");
 	
 	EidosAssertScriptSuccess_NULL("c();");
 	EidosAssertScriptSuccess_NULL("c(NULL);");
 	EidosAssertScriptSuccess_I("c(2);", 2);
 	EidosAssertScriptSuccess_IV("c(1, 2, 3);", {1, 2, 3});
-	EidosAssertScriptRaise("c(x=2);", 0, "unrecognized named argument x");
-	EidosAssertScriptRaise("c(x=1, 2, 3);", 0, "unrecognized named argument x");
-	EidosAssertScriptRaise("c(1, x=2, 3);", 0, "unrecognized named argument x");
-	EidosAssertScriptRaise("c(1, 2, x=3);", 0, "unrecognized named argument x");
+	EidosAssertScriptRaise("c(x=2);", 0, "unrecognized named argument 'x'");
+	EidosAssertScriptRaise("c(x=1, 2, 3);", 0, "unrecognized named argument 'x'");
+	EidosAssertScriptRaise("c(1, x=2, 3);", 0, "unrecognized named argument 'x'");
+	EidosAssertScriptRaise("c(1, 2, x=3);", 0, "unrecognized named argument 'x'");
 	
 	EidosAssertScriptSuccess_I("doCall('abs', -10);", 10);
 	EidosAssertScriptSuccess_I("doCall(functionName='abs', -10);", 10);
 	EidosAssertScriptRaise("doCall(x='abs', -10);", 0, "skipped over required argument");
-	EidosAssertScriptRaise("doCall('abs', x=-10);", 0, "unrecognized named argument x");
+	EidosAssertScriptRaise("doCall('abs', x=-10);", 0, "unrecognized named argument 'x'");
 	EidosAssertScriptRaise("doCall('abs', functionName=-10);", 0, "could not be matched");
 	EidosAssertScriptRaise("doCall(x='abs');", 0, "skipped over required argument");
 	EidosAssertScriptRaise("doCall(functionName='abs');", 0, "requires 1 argument(s), but 0 are supplied");
